@@ -87,7 +87,7 @@ void AbstractHighlighter::setTheme(const Theme &theme)
  * Returns the index of the first non-space character. If the line is empty,
  * or only contains white spaces, text.size() is returned.
  */
-static inline int firstNonSpaceChar(const QString &text)
+static inline int firstNonSpaceChar(QStringView text)
 {
     for (int i = 0; i < text.length(); ++i) {
         if (!text[i].isSpace()) {
@@ -97,7 +97,14 @@ static inline int firstNonSpaceChar(const QString &text)
     return text.size();
 }
 
+#if KSYNTAXHIGHLIGHTING_BUILD_DEPRECATED_SINCE(5, 87)
 State AbstractHighlighter::highlightLine(const QString &text, const State &state)
+{
+    return highlightLine(QStringView(text), state);
+}
+#endif
+
+State AbstractHighlighter::highlightLine(QStringView text, const State &state)
 {
     Q_D(AbstractHighlighter);
 
@@ -130,8 +137,7 @@ State AbstractHighlighter::highlightLine(const QString &text, const State &state
          * see https://phabricator.kde.org/D18509
          */
         int endlessLoopingCounter = 0;
-        while (!stateData->topContext()->lineEmptyContext().isStay()
-               || (stateData->topContext()->lineEmptyContext().isStay() && !stateData->topContext()->lineEndContext().isStay())) {
+        while (!stateData->topContext()->lineEmptyContext().isStay() || !stateData->topContext()->lineEndContext().isStay()) {
             /**
              * line empty context switches
              */
@@ -146,8 +152,7 @@ State AbstractHighlighter::highlightLine(const QString &text, const State &state
                  * line end context switches only when lineEmptyContext is #stay. This avoids
                  * skipping empty lines after a line continuation character (see bug 405903)
                  */
-            } else if (!stateData->topContext()->lineEndContext().isStay()
-                       && !d->switchContext(stateData, stateData->topContext()->lineEndContext(), QStringList())) {
+            } else if (!d->switchContext(stateData, stateData->topContext()->lineEndContext(), QStringList())) {
                 break;
             }
 
@@ -173,9 +178,29 @@ State AbstractHighlighter::highlightLine(const QString &text, const State &state
      *   - store the result of the first position that matches (or -1 for no match in the full line) in the skipOffsets hash for re-use
      *   - have capturesForLastDynamicSkipOffset as guard for dynamic regexes to invalidate the cache if they might have changed
      */
-    QHash<Rule *, int> skipOffsets;
+    QVarLengthArray<QPair<Rule *, int>, 8> skipOffsets;
     QStringList capturesForLastDynamicSkipOffset;
 
+    auto getSkipOffsetValue = [&skipOffsets](Rule *r) -> int {
+        auto i = std::find_if(skipOffsets.begin(), skipOffsets.end(), [r](const auto &v) {
+            return v.first == r;
+        });
+        if (i == skipOffsets.end())
+            return 0;
+        return i->second;
+    };
+
+    auto insertSkipOffset = [&skipOffsets](Rule *r, int i) {
+        auto it = std::find_if(skipOffsets.begin(), skipOffsets.end(), [r](const auto &v) {
+            return v.first == r;
+        });
+        if (it == skipOffsets.end()) {
+            skipOffsets.push_back({r, i});
+        } else {
+            it->second = i;
+        }
+    };
+
     /**
      * current active format
      * stored as pointer to avoid deconstruction/constructions inside the internal loop
@@ -250,7 +275,7 @@ State AbstractHighlighter::highlightLine(const QString &text, const State &state
             if (rule->isDynamic() && (capturesForLastDynamicSkipOffset != stateData->topCaptures())) {
                 skipOffsets.clear();
             }
-            const auto currentSkipOffset = skipOffsets.value(rule.get());
+            const auto currentSkipOffset = getSkipOffsetValue(rule.get());
             if (currentSkipOffset < 0 || currentSkipOffset > offset) {
                 continue;
             }
@@ -262,7 +287,7 @@ State AbstractHighlighter::highlightLine(const QString &text, const State &state
              * update skip offset if new one rules out any later match or is larger than current one
              */
             if (newResult.skipOffset() < 0 || newResult.skipOffset() > currentSkipOffset) {
-                skipOffsets.insert(rule.get(), newResult.skipOffset());
+                insertSkipOffset(rule.get(), newResult.skipOffset());
 
                 // remember new captures, if dynamic to enforce proper reset above on change!
                 if (rule->isDynamic()) {
@@ -298,7 +323,7 @@ State AbstractHighlighter::highlightLine(const QString &text, const State &state
 
             d->switchContext(stateData, rule->context(), newResult.captures());
             newFormat = rule->attributeFormat().isValid() ? &rule->attributeFormat() : &stateData->topContext()->attributeFormat();
-            if (newOffset == text.size() && std::dynamic_pointer_cast<LineContinue>(rule)) {
+            if (newOffset == text.size() && rule->isLineContinue()) {
                 lineContinuation = true;
             }
             break;
